@@ -1,17 +1,12 @@
-# -*- coding: utf-8 -*-
 # Unittests for fixtures.
-from __future__ import unicode_literals
-
 import json
 import os
 import re
-import unittest
-import warnings
+from io import StringIO
+from pathlib import Path
 
-import django
 from django.core import management, serializers
 from django.core.exceptions import ImproperlyConfigured
-from django.core.management.base import CommandError
 from django.core.serializers.base import DeserializationError
 from django.db import IntegrityError, transaction
 from django.db.models import signals
@@ -19,9 +14,6 @@ from django.test import (
     TestCase, TransactionTestCase, override_settings, skipIfDBFeature,
     skipUnlessDBFeature,
 )
-from django.utils import six
-from django.utils._os import upath
-from django.utils.six import PY3, StringIO
 
 from .models import (
     Absolute, Animal, Article, Book, Child, Circle1, Circle2, Circle3,
@@ -33,17 +25,7 @@ from .models import (
     Person, RefToNKChild, Store, Stuff, Thingy, Widget,
 )
 
-_cur_dir = os.path.dirname(os.path.abspath(upath(__file__)))
-
-
-def is_ascii(s):
-    return all(ord(c) < 128 for c in s)
-
-
-skipIfNonASCIIPath = unittest.skipIf(
-    not is_ascii(django.__file__) and six.PY2,
-    'Python 2 crashes when checking non-ASCII exception messages.'
-)
+_cur_dir = os.path.dirname(os.path.abspath(__file__))
 
 
 class TestFixtures(TestCase):
@@ -131,8 +113,8 @@ class TestFixtures(TestCase):
             'pretty.xml',
             verbosity=0,
         )
-        self.assertEqual(Stuff.objects.all()[0].name, None)
-        self.assertEqual(Stuff.objects.all()[0].owner, None)
+        self.assertIsNone(Stuff.objects.all()[0].name)
+        self.assertIsNone(Stuff.objects.all()[0].owner)
 
     @skipUnlessDBFeature('interprets_empty_strings_as_nulls')
     def test_pretty_print_xml_empty_strings(self):
@@ -147,7 +129,7 @@ class TestFixtures(TestCase):
             verbosity=0,
         )
         self.assertEqual(Stuff.objects.all()[0].name, '')
-        self.assertEqual(Stuff.objects.all()[0].owner, None)
+        self.assertIsNone(Stuff.objects.all()[0].owner)
 
     def test_absolute_path(self):
         """
@@ -159,7 +141,7 @@ class TestFixtures(TestCase):
         fixture directory.
         """
         load_absolute_path = os.path.join(
-            os.path.dirname(upath(__file__)),
+            os.path.dirname(__file__),
             'fixtures',
             'absolute.json'
         )
@@ -201,26 +183,23 @@ class TestFixtures(TestCase):
         Test for ticket #4371 -- Loading data of an unknown format should fail
         Validate that error conditions are caught correctly
         """
-        with six.assertRaisesRegex(self, management.CommandError,
-                "Problem installing fixture 'bad_fixture1': "
-                "unkn is not a known serialization format."):
+        msg = "Problem installing fixture 'bad_fix.ture1': unkn is not a known serialization format."
+        with self.assertRaisesMessage(management.CommandError, msg):
             management.call_command(
                 'loaddata',
-                'bad_fixture1.unkn',
+                'bad_fix.ture1.unkn',
                 verbosity=0,
             )
 
-    @skipIfNonASCIIPath
     @override_settings(SERIALIZATION_MODULES={'unkn': 'unexistent.path'})
     def test_unimportable_serializer(self):
         """
-        Test that failing serializer import raises the proper error
+        Failing serializer import raises the proper error
         """
-        with six.assertRaisesRegex(self, ImportError,
-                r"No module named.*unexistent"):
+        with self.assertRaisesMessage(ImportError, "No module named 'unexistent'"):
             management.call_command(
                 'loaddata',
-                'bad_fixture1.unkn',
+                'bad_fix.ture1.unkn',
                 verbosity=0,
             )
 
@@ -230,19 +209,13 @@ class TestFixtures(TestCase):
         using explicit filename.
         Test for ticket #18213 -- warning conditions are caught correctly
         """
-        with warnings.catch_warnings(record=True) as warning_list:
-            warnings.simplefilter("always")
+        msg = "No fixture data found for 'bad_fixture2'. (File format may be invalid.)"
+        with self.assertWarnsMessage(RuntimeWarning, msg):
             management.call_command(
                 'loaddata',
                 'bad_fixture2.xml',
                 verbosity=0,
             )
-            warning = warning_list.pop()
-            self.assertEqual(warning.category, RuntimeWarning)
-            self.assertEqual(
-                str(warning.message),
-                "No fixture data found for 'bad_fixture2'. (File format may be invalid.)"
-            )
 
     def test_invalid_data_no_ext(self):
         """
@@ -250,56 +223,40 @@ class TestFixtures(TestCase):
         without file extension.
         Test for ticket #18213 -- warning conditions are caught correctly
         """
-        with warnings.catch_warnings(record=True) as warning_list:
-            warnings.simplefilter("always")
+        msg = "No fixture data found for 'bad_fixture2'. (File format may be invalid.)"
+        with self.assertWarnsMessage(RuntimeWarning, msg):
             management.call_command(
                 'loaddata',
                 'bad_fixture2',
                 verbosity=0,
             )
-            warning = warning_list.pop()
-            self.assertEqual(warning.category, RuntimeWarning)
-            self.assertEqual(
-                str(warning.message),
-                "No fixture data found for 'bad_fixture2'. (File format may be invalid.)"
-            )
 
     def test_empty(self):
         """
         Test for ticket #18213 -- Loading a fixture file with no data output a warning.
         Previously empty fixture raises an error exception, see ticket #4371.
         """
-        with warnings.catch_warnings(record=True) as warning_list:
-            warnings.simplefilter("always")
+        msg = "No fixture data found for 'empty'. (File format may be invalid.)"
+        with self.assertWarnsMessage(RuntimeWarning, msg):
             management.call_command(
                 'loaddata',
                 'empty',
                 verbosity=0,
             )
-            warning = warning_list.pop()
-            self.assertEqual(warning.category, RuntimeWarning)
-            self.assertEqual(str(warning.message),
-            "No fixture data found for 'empty'. (File format may be invalid.)")
 
     def test_error_message(self):
         """
         Regression for #9011 - error message is correct.
         Change from error to warning for ticket #18213.
         """
-        with warnings.catch_warnings(record=True) as warning_list:
-            warnings.simplefilter("always")
+        msg = "No fixture data found for 'bad_fixture2'. (File format may be invalid.)"
+        with self.assertWarnsMessage(RuntimeWarning, msg):
             management.call_command(
                 'loaddata',
                 'bad_fixture2',
                 'animal',
                 verbosity=0,
             )
-            warning = warning_list.pop()
-            self.assertEqual(warning.category, RuntimeWarning)
-            self.assertEqual(
-                str(warning.message),
-                "No fixture data found for 'bad_fixture2'. (File format may be invalid.)"
-            )
 
     def test_pg_sequence_resetting_checks(self):
         """
@@ -357,8 +314,8 @@ class TestFixtures(TestCase):
             self.assertEqual(
                 self.pre_save_checks,
                 [
-                    ("Count = 42 (<%s 'int'>)" % ('class' if PY3 else 'type'),
-                     "Weight = 1.2 (<%s 'float'>)" % ('class' if PY3 else 'type'))
+                    ("Count = 42 (<class 'int'>)",
+                     "Weight = 1.2 (<class 'float'>)")
                 ]
             )
         finally:
@@ -367,8 +324,8 @@ class TestFixtures(TestCase):
     def test_dumpdata_uses_default_manager(self):
         """
         Regression for #11286
-        Ensure that dumpdata honors the default manager
-        Dump the current contents of the database as a JSON fixture
+        Dumpdata honors the default manager. Dump the current contents of
+        the database as a JSON fixture
         """
         management.call_command(
             'loaddata',
@@ -460,8 +417,7 @@ class TestFixtures(TestCase):
         """
         Regression for #3615 - Ensure data with nonexistent child key references raises error
         """
-        with six.assertRaisesRegex(self, IntegrityError,
-                "Problem installing fixture"):
+        with self.assertRaisesMessage(IntegrityError, "Problem installing fixture"):
             management.call_command(
                 'loaddata',
                 'forward_ref_bad_data.json',
@@ -489,9 +445,8 @@ class TestFixtures(TestCase):
         """
         Regression for #7043 - Error is quickly reported when no fixtures is provided in the command line.
         """
-        with six.assertRaisesRegex(self, management.CommandError,
-                "No database fixture specified. Please provide the path of "
-                "at least one fixture in the command line."):
+        msg = "No database fixture specified. Please provide the path of at least one fixture in the command line."
+        with self.assertRaisesMessage(management.CommandError, msg):
             management.call_command(
                 'loaddata',
                 verbosity=0,
@@ -537,32 +492,22 @@ class TestFixtures(TestCase):
         settings.FIXTURE_DIRS cannot contain duplicates in order to avoid
         repeated fixture loading.
         """
-        self.assertRaisesMessage(
-            ImproperlyConfigured,
-            "settings.FIXTURE_DIRS contains duplicates.",
-            management.call_command,
-            'loaddata',
-            'absolute.json',
-            verbosity=0,
-        )
+        with self.assertRaisesMessage(ImproperlyConfigured, "settings.FIXTURE_DIRS contains duplicates."):
+            management.call_command('loaddata', 'absolute.json', verbosity=0)
 
-    @skipIfNonASCIIPath
     @override_settings(FIXTURE_DIRS=[os.path.join(_cur_dir, 'fixtures')])
     def test_fixture_dirs_with_default_fixture_path(self):
         """
         settings.FIXTURE_DIRS cannot contain a default fixtures directory
         for application (app/fixtures) in order to avoid repeated fixture loading.
         """
-        self.assertRaisesMessage(
-            ImproperlyConfigured,
+        msg = (
             "'%s' is a default fixture directory for the '%s' app "
             "and cannot be listed in settings.FIXTURE_DIRS."
-            % (os.path.join(_cur_dir, 'fixtures'), 'fixtures_regress'),
-            management.call_command,
-            'loaddata',
-            'absolute.json',
-            verbosity=0,
+            % (os.path.join(_cur_dir, 'fixtures'), 'fixtures_regress')
         )
+        with self.assertRaisesMessage(ImproperlyConfigured, msg):
+            management.call_command('loaddata', 'absolute.json', verbosity=0)
 
     @override_settings(FIXTURE_DIRS=[os.path.join(_cur_dir, 'fixtures_1'),
                                      os.path.join(_cur_dir, 'fixtures_2')])
@@ -573,6 +518,11 @@ class TestFixtures(TestCase):
             verbosity=0,
         )
 
+    @override_settings(FIXTURE_DIRS=[Path(_cur_dir) / 'fixtures_1'])
+    def test_fixtures_dir_pathlib(self):
+        management.call_command('loaddata', 'inner/absolute.json', verbosity=0)
+        self.assertQuerysetEqual(Absolute.objects.all(), [1], transform=lambda o: o.pk)
+
 
 class NaturalKeyFixtureTests(TestCase):
 
@@ -632,8 +582,7 @@ class NaturalKeyFixtureTests(TestCase):
 
     def test_nk_on_serialize(self):
         """
-        Check that natural key requirements are taken into account
-        when serializing models
+        Natural key requirements are taken into account when serializing models.
         """
         management.call_command(
             'loaddata',
@@ -666,10 +615,8 @@ class NaturalKeyFixtureTests(TestCase):
 
     def test_dependency_sorting(self):
         """
-        Now lets check the dependency sorting explicitly
-        It doesn't matter what order you mention the models
-        Store *must* be serialized before then Person, and both
-        must be serialized before Book.
+        It doesn't matter what order you mention the models,  Store *must* be
+        serialized before then Person, and both must be serialized before Book.
         """
         sorted_deps = serializers.sort_dependencies(
             [('fixtures_regress', [Book, Person, Store])]
@@ -734,41 +681,37 @@ class NaturalKeyFixtureTests(TestCase):
         )
 
     def test_dependency_sorting_tight_circular(self):
-        self.assertRaisesMessage(
+        with self.assertRaisesMessage(
             RuntimeError,
             "Can't resolve dependencies for fixtures_regress.Circle1, "
-            "fixtures_regress.Circle2 in serialized app list.",
-            serializers.sort_dependencies,
-            [('fixtures_regress', [Person, Circle2, Circle1, Store, Book])],
-        )
+            "fixtures_regress.Circle2 in serialized app list."
+        ):
+            serializers.sort_dependencies([('fixtures_regress', [Person, Circle2, Circle1, Store, Book])])
 
     def test_dependency_sorting_tight_circular_2(self):
-        self.assertRaisesMessage(
+        with self.assertRaisesMessage(
             RuntimeError,
             "Can't resolve dependencies for fixtures_regress.Circle1, "
-            "fixtures_regress.Circle2 in serialized app list.",
-            serializers.sort_dependencies,
-            [('fixtures_regress', [Circle1, Book, Circle2])],
-        )
+            "fixtures_regress.Circle2 in serialized app list."
+        ):
+            serializers.sort_dependencies([('fixtures_regress', [Circle1, Book, Circle2])])
 
     def test_dependency_self_referential(self):
-        self.assertRaisesMessage(
+        with self.assertRaisesMessage(
             RuntimeError,
             "Can't resolve dependencies for fixtures_regress.Circle3 in "
-            "serialized app list.",
-            serializers.sort_dependencies,
-            [('fixtures_regress', [Book, Circle3])],
-        )
+            "serialized app list."
+        ):
+            serializers.sort_dependencies([('fixtures_regress', [Book, Circle3])])
 
     def test_dependency_sorting_long(self):
-        self.assertRaisesMessage(
+        with self.assertRaisesMessage(
             RuntimeError,
             "Can't resolve dependencies for fixtures_regress.Circle1, "
             "fixtures_regress.Circle2, fixtures_regress.Circle3 in serialized "
-            "app list.",
-            serializers.sort_dependencies,
-            [('fixtures_regress', [Person, Circle2, Circle1, Circle3, Store, Book])],
-        )
+            "app list."
+        ):
+            serializers.sort_dependencies([('fixtures_regress', [Person, Circle2, Circle1, Circle3, Store, Book])])
 
     def test_dependency_sorting_normal(self):
         sorted_deps = serializers.sort_dependencies(
@@ -781,8 +724,7 @@ class NaturalKeyFixtureTests(TestCase):
 
     def test_normal_pk(self):
         """
-        Check that normal primary keys still work
-        on a model with natural key capabilities
+        Normal primary keys work on a model with natural key capabilities.
         """
         management.call_command(
             'loaddata',
@@ -800,11 +742,12 @@ class NaturalKeyFixtureTests(TestCase):
             verbosity=0,
         )
         books = Book.objects.all()
-        self.assertEqual(
-            books.__repr__(),
-            "[<Book: Cryptonomicon by Neal Stephenson (available at Amazon, Borders)>, "
-            "<Book: Ender's Game by Orson Scott Card (available at Collins Bookstore)>, "
-            "<Book: Permutation City by Greg Egan (available at Angus and Robertson)>]"
+        self.assertQuerysetEqual(
+            books, [
+                "<Book: Cryptonomicon by Neal Stephenson (available at Amazon, Borders)>",
+                "<Book: Ender's Game by Orson Scott Card (available at Collins Bookstore)>",
+                "<Book: Permutation City by Greg Egan (available at Angus and Robertson)>",
+            ]
         )
 
 
@@ -829,13 +772,12 @@ class M2MNaturalKeyFixtureTests(TestCase):
         Resolving circular M2M relations without explicit through models should
         fail loudly
         """
-        self.assertRaisesMessage(
+        with self.assertRaisesMessage(
             RuntimeError,
             "Can't resolve dependencies for fixtures_regress.M2MSimpleCircularA, "
-            "fixtures_regress.M2MSimpleCircularB in serialized app list.",
-            serializers.sort_dependencies,
-            [('fixtures_regress', [M2MSimpleCircularA, M2MSimpleCircularB])]
-        )
+            "fixtures_regress.M2MSimpleCircularB in serialized app list."
+        ):
+            serializers.sort_dependencies([('fixtures_regress', [M2MSimpleCircularA, M2MSimpleCircularB])])
 
     def test_dependency_sorting_m2m_complex(self):
         """
@@ -856,15 +798,9 @@ class M2MNaturalKeyFixtureTests(TestCase):
         A, B, C, AtoB, BtoC, CtoA = (M2MComplexCircular1A, M2MComplexCircular1B,
                                      M2MComplexCircular1C, M2MCircular1ThroughAB,
                                      M2MCircular1ThroughBC, M2MCircular1ThroughCA)
-        try:
-            sorted_deps = serializers.sort_dependencies(
-                [('fixtures_regress', [A, B, C, AtoB, BtoC, CtoA])]
-            )
-        except CommandError:
-            self.fail("Serialization dependency solving algorithm isn't "
-                      "capable of handling circular M2M setups with "
-                      "intermediate models.")
-
+        sorted_deps = serializers.sort_dependencies(
+            [('fixtures_regress', [A, B, C, AtoB, BtoC, CtoA])]
+        )
         # The dependency sorting should not result in an error, and the
         # through model should have dependencies to the other models and as
         # such come last in the list.
@@ -876,17 +812,9 @@ class M2MNaturalKeyFixtureTests(TestCase):
         Circular M2M relations with explicit through models should be serializable
         This test tests the circularity with explicit natural_key.dependencies
         """
-        try:
-            sorted_deps = serializers.sort_dependencies([
-                ('fixtures_regress', [
-                    M2MComplexCircular2A,
-                    M2MComplexCircular2B,
-                    M2MCircular2ThroughAB])
-            ])
-        except CommandError:
-            self.fail("Serialization dependency solving algorithm isn't "
-                      "capable of handling circular M2M setups with "
-                      "intermediate models plus natural key dependency hints.")
+        sorted_deps = serializers.sort_dependencies([
+            ('fixtures_regress', [M2MComplexCircular2A, M2MComplexCircular2B, M2MCircular2ThroughAB])
+        ])
         self.assertEqual(sorted_deps[:2], [M2MComplexCircular2A, M2MComplexCircular2B])
         self.assertEqual(sorted_deps[2:], [M2MCircular2ThroughAB])
 
@@ -925,15 +853,11 @@ class M2MNaturalKeyFixtureTests(TestCase):
 
 class TestTicket11101(TransactionTestCase):
 
-    available_apps = [
-        'fixtures_regress',
-        'django.contrib.auth',
-        'django.contrib.contenttypes',
-    ]
+    available_apps = ['fixtures_regress']
 
     @skipUnlessDBFeature('supports_transactions')
     def test_ticket_11101(self):
-        """Test that fixtures can be rolled back (ticket #11101)."""
+        """Fixtures can be rolled back (ticket #11101)."""
         with transaction.atomic():
             management.call_command(
                 'loaddata',
